@@ -67,7 +67,7 @@ qos_filter_dtor(struct st_mysqlnd_ms_filter_data * pDest TSRMLS_DC)
 MYSQLND_MS_FILTER_DATA *
 mysqlnd_ms_qos_filter_ctor(struct st_mysqlnd_ms_config_json_entry * section, zend_llist * master_connections, zend_llist * slave_connections, MYSQLND_ERROR_INFO * error_info, zend_bool persistent TSRMLS_DC)
 {
-	MYSQLND_MS_FILTER_QOS_DATA * ret;
+	MYSQLND_MS_FILTER_QOS_DATA * ret = NULL;
 	DBG_ENTER("mysqlnd_ms_qos_filter_ctor");
 	DBG_INF_FMT("section=%p", section);
 	if (section) {
@@ -218,7 +218,11 @@ mysqlnd_ms_qos_server_has_gtid(MYSQLND_CONN_DATA * conn, MYSQLND_MS_CONN_DATA **
 				*/
 				DBG_INF_FMT("sleep and retry, time left=" MYSQLND_LLU_SPEC, (my_wait_time - total_time));
 				MS_TIME_SET(run_time);
+#ifdef PHP_WIN32
+				Sleep(1);
+#else
 				sleep(1);
+#endif
 				if (res) {
 					res->m.free_result(res, FALSE TSRMLS_CC);
 				}
