@@ -18,7 +18,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: php_mysqlnd_ms.c 329770 2013-03-13 15:28:49Z uw $ */
+/* $Id$ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -75,6 +75,7 @@ const MYSQLND_STRING mysqlnd_ms_stats_values_names[MS_STAT_LAST] =
 	{ STR_W_LEN("gtid_implicit_commit_injections_success") },
 	{ STR_W_LEN("gtid_implicit_commit_injections_failure") },
 #endif
+	{ STR_W_LEN("transient_error_retries") },
 };
 /* }}} */
 
@@ -275,6 +276,48 @@ static PHP_FUNCTION(mysqlnd_ms_match_wild)
 /* }}} */
 
 #if PHP_VERSION_ID > 50399
+
+#if PHP_VERSION_ID < 50418 || PHP_VERSION_ID == 50500
+static MYSQLND *zval_to_mysqlnd_inherited(zval *zv TSRMLS_DC) /* {{{ */
+{
+	zend_class_entry *root_ce;
+	
+	if (Z_TYPE_P(zv) != IS_OBJECT || !Z_OBJCE_P(zv)->parent) {
+		/* This is not an object or it is a non-inherited object, we can use the default implementation without hacks */
+		return zval_to_mysqlnd(zv TSRMLS_CC);
+	}
+	
+	root_ce = Z_OBJCE_P(zv)->parent;
+	while (root_ce->parent) {
+		root_ce = root_ce->parent;
+	}
+	
+	if (root_ce->type != ZEND_INTERNAL_CLASS) {
+		/* This can neither be mysqli nor pdo */
+		return NULL;
+	}
+	
+	if (!strcmp("mysqli", root_ce->name) || !strcmp("PDO", root_ce->name)) {
+		MYSQLND *retval;
+		zend_class_entry *orig_ce = Z_OBJCE_P(zv);
+		zend_object *object = zend_object_store_get_object(zv TSRMLS_CC);
+		object->ce = root_ce;
+		retval = zval_to_mysqlnd(zv TSRMLS_CC);
+		object->ce = orig_ce;
+		return retval;
+	} else {
+		return zval_to_mysqlnd(zv TSRMLS_CC);
+	}
+}
+/* }}} */
+#else /* PHP_VERSION_ID < 50418 || PHP_VERSION_ID == 50500 */
+static MYSQLND *zval_to_mysqlnd_inherited(zval *zv TSRMLS_DC) /* {{{ */
+{
+	return zval_to_mysqlnd(zv TSRMLS_CC);
+}
+/* }}} */
+#endif
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlnd_ms_get_last_used_connection, 0, 0, 1)
 	ZEND_ARG_INFO(0, object)
 ZEND_END_ARG_INFO()
@@ -290,7 +333,7 @@ static PHP_FUNCTION(mysqlnd_ms_get_last_used_connection)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &handle) == FAILURE) {
 		return;
 	}
-	if (!(proxy_conn = zval_to_mysqlnd(handle TSRMLS_CC))) {
+	if (!(proxy_conn = zval_to_mysqlnd_inherited(handle TSRMLS_CC))) {
 		RETURN_FALSE;
 	}
 	{
@@ -330,7 +373,7 @@ static PHP_FUNCTION(mysqlnd_ms_get_last_gtid)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &handle) == FAILURE) {
 		return;
 	}
-	if (!(proxy_conn = zval_to_mysqlnd(handle TSRMLS_CC))) {
+	if (!(proxy_conn = zval_to_mysqlnd_inherited(handle TSRMLS_CC))) {
 		RETURN_FALSE;
 	}
 	{
@@ -413,7 +456,7 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlnd_ms_set_qos, 0, 0, 2)
 ZEND_END_ARG_INFO()
 
 
-/* {{{ proto bool mysqlnd_ms_set_qos)
+/* {{{ proto bool mysqlnd_ms_set_qos()
    */
 static PHP_FUNCTION(mysqlnd_ms_set_qos)
 {
@@ -503,7 +546,7 @@ static PHP_FUNCTION(mysqlnd_ms_set_qos)
 		option = QOS_OPTION_NONE;
 	}
 
-	if (!(proxy_conn = zval_to_mysqlnd(handle TSRMLS_CC))) {
+	if (!(proxy_conn = zval_to_mysqlnd_inherited(handle TSRMLS_CC))) {
 		RETURN_FALSE;
 	}
 
@@ -554,7 +597,7 @@ static PHP_FUNCTION(mysqlnd_ms_set_qos)
 
 	RETURN_FALSE;
 }
-
+/* }}} */
 #endif
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlnd_ms_query_is_select, 0, 0, 1)
@@ -602,6 +645,200 @@ static PHP_FUNCTION(mysqlnd_ms_get_stats)
 }
 /* }}} */
 
+static void mysqlnd_ms_fabric_select_servers(zval *return_value, zval *conn_zv, char *table, char *key, enum mysqlnd_fabric_hint hint TSRMLS_DC) /* {{{ */
+{
+	MYSQLND *proxy_conn;
+	MYSQLND_MS_CONN_DATA **conn_data = NULL;
+	mysqlnd_fabric_server *servers, *tofree;
+
+	if (!(proxy_conn = zval_to_mysqlnd_inherited(conn_zv TSRMLS_CC))) {
+		RETURN_FALSE;
+	}
+
+	conn_data = (MYSQLND_MS_CONN_DATA **) mysqlnd_plugin_get_plugin_connection_data_data(proxy_conn->data, mysqlnd_ms_plugin_id);
+	if (!conn_data || !(*conn_data)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, MYSQLND_MS_ERROR_PREFIX " No mysqlnd_ms connection");
+		RETURN_FALSE;
+	}
+	
+	if (!(*conn_data)->fabric) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Connection is not configured to use MySQL Fabric");
+		RETURN_FALSE;
+	}
+
+	zend_llist_clean(&(*conn_data)->master_connections);
+	zend_llist_clean(&(*conn_data)->slave_connections);
+	
+	tofree = servers = mysqlnd_fabric_get_shard_servers((*conn_data)->fabric, table, key, hint);
+	if (!servers) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Didn't receive usable servers from MySQL Fabric");
+		RETURN_FALSE;
+	}
+	
+	for (; servers->hostname; servers++) {
+		MYSQLND *conn = mysqlnd_init(proxy_conn->data->persistent);
+		
+		if (servers->master) {
+			mysqlnd_ms_connect_to_host_aux(proxy_conn->data, conn->data, servers->hostname, TRUE,  servers->hostname, servers->port, &(*conn_data)->master_connections, &(*conn_data)->cred, &(*conn_data)->global_trx, TRUE, proxy_conn->data->persistent TSRMLS_CC);
+		} else {
+			mysqlnd_ms_connect_to_host_aux(proxy_conn->data, conn->data, servers->hostname, FALSE, servers->hostname, servers->port, &(*conn_data)->slave_connections,  &(*conn_data)->cred, &(*conn_data)->global_trx, TRUE, proxy_conn->data->persistent TSRMLS_CC);
+		}
+
+		conn->m->dtor(conn TSRMLS_CC);
+	}
+	
+	mysqlnd_fabric_free_server_list(tofree);
+	
+	RETURN_TRUE;
+}
+/* }}} */
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlnd_ms_fabric_select_shard, 0, 0, 3)
+	ZEND_ARG_INFO(0, connection)
+    ZEND_ARG_INFO(0, table)
+	ZEND_ARG_INFO(0, shard_key)
+ZEND_END_ARG_INFO()
+
+/* {{{ proto long mysqlnd_ms_fabric_select_shard(mixed connection, string table, string shard_key)
+   Pick server configuration for a shard key */
+static PHP_FUNCTION(mysqlnd_ms_fabric_select_shard)
+{
+	zval *conn_zv;
+	char *table, *key;
+	int table_len, key_len;
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zss", &conn_zv, &table, &table_len, &key, &key_len) == FAILURE) {
+		return;
+	}
+
+	mysqlnd_ms_fabric_select_servers(return_value, conn_zv, table, key, LOCAL TSRMLS_CC);
+}
+/* }}} */
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlnd_ms_fabric_select_global, 0, 0, 2)
+	ZEND_ARG_INFO(0, connection)
+    ZEND_ARG_INFO(0, table)
+ZEND_END_ARG_INFO()
+
+/* {{{ proto long mysqlnd_ms_fabric_select_global(mixed connection, string table)
+   Pick server configuration for a shard key */
+static PHP_FUNCTION(mysqlnd_ms_fabric_select_global)
+{
+	zval *conn_zv;
+	char *table;
+	int table_len;
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zs", &conn_zv, &table, &table_len) == FAILURE) {
+		return;
+	}
+
+	mysqlnd_ms_fabric_select_servers(return_value, conn_zv, table, NULL, GLOBAL TSRMLS_CC);	
+}
+/* }}} */
+
+static void mysqlnd_ms_add_server_to_array(void *data, void *arg TSRMLS_DC) /* {{{ */
+{
+	zval *host;
+	MYSQLND_MS_LIST_DATA **element = (MYSQLND_MS_LIST_DATA **)data;
+	zval *array = (zval *)arg;
+	
+	MAKE_STD_ZVAL(host);
+	array_init(host);
+	add_assoc_string(host, "hostname", (*element)->host, 1);
+	if ((*element)->socket) {
+		add_assoc_string(host, "socket", (*element)->socket, 1);
+	}		
+	add_assoc_long(host, "port", (*element)->port);
+	
+	add_next_index_zval(array, host);
+}
+/* }}} */
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_mysqlnd_ms_dump_servers, 0, 0, 1)
+	ZEND_ARG_INFO(0, connection)
+ZEND_END_ARG_INFO()
+
+/* {{{ proto long mysqlnd_ms_dump_servers(mixed connection)
+   Dump configured master and slave servers */
+static PHP_FUNCTION(mysqlnd_ms_dump_servers)
+{
+	zval *conn_zv, *master, *slaves;
+	MYSQLND *conn;
+	MYSQLND_MS_CONN_DATA **conn_data = NULL;
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &conn_zv) == FAILURE) {
+		return;
+	}
+
+	if (!(conn = zval_to_mysqlnd_inherited(conn_zv TSRMLS_CC))) {
+		RETURN_FALSE;
+	}
+
+	conn_data = (MYSQLND_MS_CONN_DATA **) mysqlnd_plugin_get_plugin_connection_data_data(conn->data, mysqlnd_ms_plugin_id);
+	if (!conn_data || !(*conn_data)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, MYSQLND_MS_ERROR_PREFIX " No mysqlnd_ms connection");
+		RETURN_FALSE;
+	}
+	
+	MAKE_STD_ZVAL(master);
+	MAKE_STD_ZVAL(slaves);
+	array_init(master);
+	array_init(slaves);
+	
+	zend_llist_apply_with_argument(&(*conn_data)->master_connections, mysqlnd_ms_add_server_to_array, master);
+	zend_llist_apply_with_argument(&(*conn_data)->slave_connections, mysqlnd_ms_add_server_to_array, slaves);
+	
+	array_init(return_value);
+	add_assoc_zval(return_value, "master", master);
+	add_assoc_zval(return_value, "slaves", slaves);
+}
+/* }}} */
+
+static void mysqlnd_ms_dump_fabric_hosts_cb(const char *hostname, unsigned int port, void *data) /* {{{ */
+{
+	zval *item;
+	zval *return_value = (zval*)data;
+	
+	MAKE_STD_ZVAL(item);
+	array_init(item);
+	add_assoc_string(item, "hostname", hostname, 1);
+	add_assoc_long(item, "port", port);
+	
+	add_next_index_zval(return_value,  item);
+}
+/* }}} */
+
+/* {{{ proto long mysqlnd_ms_dump_fabric_hosts(mixed connection)
+   Dump configured master and slave servers */
+static PHP_FUNCTION(mysqlnd_ms_dump_fabric_hosts)
+{
+	zval *conn_zv;
+	MYSQLND *conn;
+	MYSQLND_MS_CONN_DATA **conn_data = NULL;
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &conn_zv) == FAILURE) {
+		return;
+	}
+
+	if (!(conn = zval_to_mysqlnd_inherited(conn_zv TSRMLS_CC))) {
+		RETURN_FALSE;
+	}
+
+	conn_data = (MYSQLND_MS_CONN_DATA **) mysqlnd_plugin_get_plugin_connection_data_data(conn->data, mysqlnd_ms_plugin_id);
+	if (!conn_data || !(*conn_data)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, MYSQLND_MS_ERROR_PREFIX " No mysqlnd_ms connection");
+		RETURN_FALSE;
+	}
+	
+	if (!(*conn_data)->fabric) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, MYSQLND_MS_ERROR_PREFIX " No MySQL Fabric connection");
+		RETURN_FALSE;
+	}
+
+	array_init(return_value);
+	mysqlnd_fabric_host_list_apply((*conn_data)->fabric, mysqlnd_ms_dump_fabric_hosts_cb, return_value);
+}
+/* }}} */
 
 /* {{{ mysqlnd_ms_deps[] */
 static const zend_module_dep mysqlnd_ms_deps[] = {
@@ -629,6 +866,10 @@ static const zend_function_entry mysqlnd_ms_functions[] = {
 	PHP_FE(mysqlnd_ms_get_last_gtid,	arginfo_mysqlnd_ms_get_last_gtid)
 	PHP_FE(mysqlnd_ms_set_qos,	arginfo_mysqlnd_ms_set_qos)
 #endif
+	PHP_FE(mysqlnd_ms_fabric_select_shard, arginfo_mysqlnd_ms_fabric_select_shard)
+	PHP_FE(mysqlnd_ms_fabric_select_global, arginfo_mysqlnd_ms_fabric_select_global)
+	PHP_FE(mysqlnd_ms_dump_servers, arginfo_mysqlnd_ms_dump_servers)
+	PHP_FE(mysqlnd_ms_dump_fabric_hosts, arginfo_mysqlnd_ms_dump_servers)
 	{NULL, NULL, NULL}	/* Must be the last line in mysqlnd_ms_functions[] */
 };
 /* }}} */
